@@ -10,12 +10,14 @@
 import os
 import re
 import shutil
+import subprocess
 import sys
 import yaml
 
 from datetime       import datetime
 
 from mflowgen.utils import bold, yellow
+from mflowgen.utils import read_yaml, write_yaml
 
 #-------------------------------------------------------------------------
 # Stash Management
@@ -68,17 +70,6 @@ class StashHandler:
       'help',
     ]
 
-    # Helper function
-
-    def read_yaml( yaml_path ):
-      with open( yaml_path ) as fd:
-        try:
-          data = yaml.load( fd, Loader=yaml.FullLoader )
-        except AttributeError:
-          # PyYAML for python2 does not have FullLoader
-          data = yaml.load( fd )
-      return data
-
     # Read YAML: Grab link path from hidden YAML
 
     s.link_path_yaml = '.mflowgen.stash.yml'
@@ -184,24 +175,30 @@ class StashHandler:
 
   def set_stash_path( s, path ):
     s.link_path = path
-    with open( s.link_path_yaml, 'w' ) as fd:
-      yaml.dump( { 'path' : s.link_path }, fd, default_flow_style=False )
+    write_yaml(
+      data = { 'path' : s.link_path },
+      path = s.link_path_yaml,
+    )
 
   #-----------------------------------------------------------------------
   # update_stash
   #-----------------------------------------------------------------------
   # Update the metadata in the linked stash directory
+  #
 
   def update_stash( s ):
-    with open( s.stash_yaml_path, 'w' ) as fd:
-      yaml.dump( s.stash, fd, default_flow_style=False )
+    write_yaml(
+      data = s.stash,
+      path = s.stash_yaml_path,
+    )
 
   #-----------------------------------------------------------------------
   # launch
   #-----------------------------------------------------------------------
   # Dispatch function for commands
+  #
 
-  def launch( s, args, help_, path, step, msg, hash_ ):
+  def launch( s, args, help_, path, step, msg, hash_, all_, verbose ):
 
     if help_ and not args:
       s.launch_help()
@@ -225,8 +222,8 @@ class StashHandler:
 
     if   command == 'init' : s.launch_init( help_, path )
     elif command == 'link' : s.launch_link( help_, path )
-    elif command == 'list' : s.launch_list( help_ )
-    elif command == 'push' : s.launch_push( help_, step, msg )
+    elif command == 'list' : s.launch_list( help_, verbose, all_ )
+    elif command == 'push' : s.launch_push( help_, step, msg, all_ )
     elif command == 'pull' : s.launch_pull( help_, hash_ )
     elif command == 'pop'  : s.launch_pop ( help_, hash_ )
     elif command == 'drop' : s.launch_drop( help_, hash_ )
@@ -330,16 +327,18 @@ class StashHandler:
   # - Reads the metadata YAML in the stash directory to list the stash
   #
 
-  def launch_list( s, help_ ):
+  def launch_list( s, help_, verbose, all_ ):
 
     # Help message
 
     def print_help():
       print()
-      print( bold( 'Usage:' ), 'mflowgen stash list'                  )
+      print( bold( 'Usage:' ), 'mflowgen stash list [--verbose] [--all]' )
       print()
-      print( 'Lists all pre-built steps stored in the mflowgen stash' )
-      print( 'that the current build graph is linked to.'             )
+      print( 'Lists all pre-built steps stored in the mflowgen stash'    )
+      print( 'that the current build graph is linked to. The --verbose'  )
+      print( 'flag prints metadata about where each stashed step was'    )
+      print( 'stashed from. Use --all to print all steps in the stash.'  )
       print()
 
     if help_:
@@ -358,11 +357,17 @@ class StashHandler:
     template_str = \
       ' - {hash_} [ {date} ] {author} {step} -- {msg}'
 
+    stashed_from_template_str = \
+      '     > {k:30} : {v}'
+
     print()
     if not s.stash:
       print( ' - ( the stash is empty )' )
     else:
-      for x in s.stash:
+      s.stash.reverse()  # print in reverse chronological order
+      n_print = 10       # print first N items
+      to_print = s.stash[:n_print] if not all_ else s.stash
+      for x in to_print:
         print( template_str.format(
           hash_  = yellow( x[ 'hash' ] ),
           date   = x[ 'date'   ],
@@ -370,6 +375,13 @@ class StashHandler:
           step   = x[ 'step'   ],
           msg    = x[ 'msg'    ],
         ) )
+        if verbose and 'stashed-from' in x.keys(): # stashed from
+          for k, v in x['stashed-from'].items():
+            print( stashed_from_template_str.format(k=k,v=v) )
+          print()
+      if not all_ and len(s.stash) > n_print:
+        n_extra = len(s.stash) - n_print
+        print( ' - (...) see', n_extra, 'more with --all' )
     print()
     print( bold( 'Stash:' ), s.get_stash_path() )
     print()
@@ -380,10 +392,11 @@ class StashHandler:
   # Internally, this command does the following:
   #
   # - Copies the target step build directory to the stash
+  #     - if "all_" is True, then stash the entire step, not just outputs
   # - Updates the metadata YAML in the stash directory
   #
 
-  def launch_push( s, help_, step, msg ):
+  def launch_push( s, help_, step, msg, all_ ):
 
     try:
       author = os.environ[ 'USER' ]
@@ -395,20 +408,22 @@ class StashHandler:
     def print_help():
       print()
       print( bold( 'Usage:' ), 'mflowgen stash push',
-                                  '--step/-s <int> --message/-m "<str>"' )
+                                  '--step/-s <int> --message/-m "<str>"',
+                                  '[--all]'                              )
       print()
       print( bold( 'Example:' ), 'mflowgen stash push',
                                     '--step 5 -m "foo bar"'              )
       print()
       print( 'Pushes a built step to the mflowgen stash. The given step' )
-      print( 'is copied to the stash using archive mode (preserves all'  )
-      print( 'permissions) while following all symlinks. Then the'       )
-      print( 'stashed copy is given a hash stamp and is marked as'       )
-      print( 'authored by $USER (' + author + '). An optional message'   )
-      print( 'can also be attached to the push.'                         )
+      print( 'is copied to the stash, preserving all permissions and'    )
+      print( 'following all symlinks. By default, only the outputs of a' )
+      print( 'step are stashed, but the entire step can be stashed with' )
+      print( '--all. The stashed copy is given a hash stamp and is'      )
+      print( 'marked as authored by $USER (' + author + '). An optional' )
+      print( 'message can also be attached to each push.'                )
       print()
 
-    if help_ or not step or not msg:
+    if help_ or step==None or not msg:
       print_help()
       return
 
@@ -446,12 +461,75 @@ class StashHandler:
 
     dst_dirname = '-'.join( [ datestamp, step_name, hashstamp ] )
 
+    # Try to get some information to help describe "where this step came
+    # from"
+
+    def get_shell_output( cmd ):
+      try:
+        output = subprocess.check_output( cmd.split(),
+                                          stderr=subprocess.DEVNULL,
+                                          universal_newlines=True )
+        output = output.strip()
+      except Exception:
+        output = ''
+      return output
+
+    def get_hostname():
+      import socket
+      return socket.gethostname()
+
+    git_cmd  = 'git rev-parse --short HEAD'    # git commit hash
+    git_hash = get_shell_output( git_cmd )
+
+    git_cmd  = 'git rev-parse --show-toplevel' # git root dir
+    git_repo = get_shell_output( git_cmd )
+    git_repo = os.path.basename( git_repo )
+
+    build_path = os.getcwd()                   # build dir path
+    hostname   = get_hostname()                # hostname
+
+    stashed_from = {
+      'stashed-from-git-root-dir'      : git_repo,
+      'stashed-from-git-root-dir-hash' : git_hash,
+      'stashed-from-dir'               : build_path,
+      'stashed-from-hostname'          : hostname,
+    }
+
+    # Helper function to ignore copying files other than the outputs
+
+    def f_ignore( path, files ):
+      # For nested subdirectories, ignore all files
+      if '/' in path:
+        if path.split('/')[1] == 'outputs':
+          return []     # ignore nothing in outputs
+        elif path.split('/')[1] in [ 'logs', 'reports' ]: # TEMPORARY
+          return []     # ignore nothing                  # TEMPORARY
+        else:
+          return files  # ignore everything for any other directory
+      # At the top level, keep the outputs and a few other misc files
+      keep = [
+        'outputs',
+        'logs',      # TEMPORARY
+        'reports',   # TEMPORARY
+        'configure.yml',
+        'mflowgen-run.log',
+        '.time_start',
+        '.time_end',
+        '.stamp',
+        '.execstamp',
+        '.postconditions.stamp',
+      ]
+      ignore = [ _ for _ in files if _ not in keep ]
+      return ignore
+
     # Now copy src to dst
     #
-    # - symlinks                 = False # Follow all symlinks
-    # - ignore_dangling_symlinks = False # Stop with error if we cannot
-    #                                    #  follow a link to something
-    #                                    #  we need
+    # - symlinks                 = False  # Follow all symlinks
+    # - ignore_dangling_symlinks = False  # Stop with error if we cannot
+    #                                     #  follow a link to something
+    #                                     #  we need
+    # - ignore                   = (func) # Ignore all but the outputs
+    #                                     #  unless "--all" was given
     #
 
     remote_path = s.get_stash_path() + '/' + dst_dirname
@@ -460,6 +538,7 @@ class StashHandler:
       shutil.copytree( src      = push_target,
                        dst      = remote_path,
                        symlinks = False,
+                       ignore   = None if all_ else f_ignore,
                        ignore_dangling_symlinks = False )
     except shutil.Error:
       # According to online discussion, ignore_dangling_symlinks does not
@@ -492,17 +571,32 @@ class StashHandler:
     # Update the metadata in the stash
 
     push_metadata = {
-      'date'   : datestamp,
-      'dir'    : dst_dirname,
-      'hash'   : hashstamp,
-      'author' : author,
-      'step'   : step_name,
-      'msg'    : msg,
+      'date'         : datestamp,
+      'dir'          : dst_dirname,
+      'hash'         : hashstamp,
+      'author'       : author,
+      'step'         : step_name,
+      'msg'          : msg,
+      'stashed-from' : stashed_from,
     }
 
     s.stash.append( push_metadata )
     s.update_stash()
 
+    # Try adding the metadata to the stashed step itself, so that when the
+    # step gets pulled somewhere, we have all of its metadata to know
+    # where it came from
+
+    try:
+      data = push_metadata
+      data.update( { 'stash-dir': s.link_path } ) # add stash dir
+      write_yaml(
+        data = data,
+        path = remote_path+'/.mflowgen.stash.node.yml',
+      )
+    except Exception as e:
+      pass
+
     print(
       'Stashed step {step} "{step_name}" as author "{author}"'.format(
       step      = step,
