@@ -7,8 +7,7 @@
 # is intended to be used to define common process, technology, and library
 # information. Then innovus_config.tcl is then used to define
 # "block-specific" information like power and ground nets, tie and filler
-# cells, welltaps and endcaps, enabling useful skew, enabling clock
-# gating, etc.
+# cells, welltaps and endcaps, enabling clock gating, etc.
 #
 # In our BRG flow, most of our blocks are built the same way, so most of
 # the "block-specific" information is really common information. For
@@ -77,6 +76,9 @@ set vars(library_sets)        "libs_typical"
 
 set vars(libs_typical,timing) [join "
                                 $vars(adk_dir)/stdcells.lib
+                                [glob -nocomplain $vars(adk_dir)/stdcells-lvt.lib]
+                                [glob -nocomplain $vars(adk_dir)/stdcells-ulvt.lib]
+                                [glob -nocomplain $vars(adk_dir)/stdcells-pm.lib]
                                 [glob -nocomplain $vars(adk_dir)/iocells.lib]
                                 [glob -nocomplain inputs/*tt*.lib]
                                 [glob -nocomplain inputs/*TT*.lib]
@@ -91,7 +93,11 @@ set vars(libs_typical,timing) [join "
 if {[file exists $vars(adk_dir)/stdcells-bc.lib]} {
   set vars(libs_bc,timing)    [join "
                                 $vars(adk_dir)/stdcells-bc.lib
+                                [glob -nocomplain $vars(adk_dir)/stdcells-lvt-bc.lib]
+                                [glob -nocomplain $vars(adk_dir)/stdcells-ulvt-bc.lib]
+                                [glob -nocomplain $vars(adk_dir)/stdcells-pm-bc.lib]
                                 [glob -nocomplain $vars(adk_dir)/iocells-bc.lib]
+                                [glob -nocomplain $vars(adk_dir)/*-bc*.lib]
                                 [glob -nocomplain inputs/*ff*.lib]
                                 [glob -nocomplain inputs/*FF*.lib]
                               "]
@@ -117,6 +123,7 @@ if {[file exists $vars(adk_dir)/stdcells-wc.lib]} {
 set vars(lef_files) [join "
                       $vars(adk_dir)/rtk-tech.lef
                       $vars(adk_dir)/stdcells.lef
+                      [glob -nocomplain $vars(adk_dir)/stdcells-pm.lef]
                       [glob -nocomplain $vars(adk_dir)/*.lef]
                       [glob -nocomplain inputs/*.lef]
                     "]
@@ -378,6 +385,10 @@ set vars(signoff,stream_out,replace_tcl)      $vars(custom_scripts_dir)/stream-o
 
 set vars(signoff,summary_report,replace_tcl)  $vars(custom_scripts_dir)/summary-report.tcl
 
+# CCOPT mode options are all during cts
+
+set vars(cts,set_ccopt_mode,skip) true
+
 #-------------------------------------------------------------------------
 # Misc
 #-------------------------------------------------------------------------
@@ -440,11 +451,6 @@ set vars(filler_cells)           $ADK_FILLER_CELLS
 
 set vars(antenna_diode)          $ADK_ANTENNA_CELL
 
-# List of buffers to use during useful skew
-
-set vars(useful_skew)  true
-#set vars(skew_buffers) ""
-
 # Clock-gate aware
 
 set vars(clock_gate_aware) true
@@ -496,7 +502,6 @@ set vars(time_design_options,hold)     -expandedViews
 
 set vars(flow_effort)                  standard
 set vars(congestion_effort)            medium
-set vars(ccopt_effort)                 high
 set vars(power_effort)                 high
 set vars(multi_cut_effort)             high
 
@@ -517,17 +522,21 @@ if {$captable_only_mode} {
 set vars(tags,verbose)                 true
 set vars(tags,verbosity_level)         high
 
+# Skip verifyConnectivity to avoid out-of-memory crash on some large designs
+
+if { $::env(skip_verify_connectivity) } {
+  set vars(signoff,verify_connectivity,skip) true
+}
+
 #-------------------------------------------------------------------------
 # Reduced-effort flow that sacrifices timing to iterate more quickly
 #-------------------------------------------------------------------------
 
 if { $::env(express_flow) } {
-  set vars(flow_effort)       express
-  set vars(congestion_effort) low
-  set vars(ccopt_effort)      low
-  set vars(power_effort)      low
-  set vars(multi_cut_effort)  low
-
+  set vars(flow_effort)                 express
+  set vars(congestion_effort)           low
+  set vars(power_effort)                low
+  set vars(multi_cut_effort)            low
   set vars(postroute_extraction_effort) medium
 
   # Skipping (see "Tags for Innovus Flow")
@@ -555,3 +564,7 @@ if { $::env(express_flow) } {
 #  set vars(signoff,time_design_hold,skip) true
 }
 
+# Allows user to override any of the defaults in this file
+if [file exists inputs/setup.tcl] {
+  source inputs/setup.tcl
+}
