@@ -17,7 +17,10 @@
 
 set ptpx_additional_search_path   inputs/adk
 set ptpx_target_libraries         inputs/adk/stdcells.db
-set ptpx_extra_link_libraries     [glob -nocomplain inputs/*.db]
+set ptpx_extra_link_libraries     [join "
+                                    [glob -nocomplain inputs/*.db]
+                                    [glob -nocomplain inputs/adk/*.db]
+                                "]
 
 #-------------------------------------------------------------------------
 # Interface to the build system
