@@ -14,9 +14,7 @@
 
 update_timing -full
 
-extract_model -library_cell -output ${ptpx_design_name} -format {lib db} -block_scope
+extract_model -library_cell -output ${ptpx_design_name} -format {lib db}
 
 file rename -force ${ptpx_design_name}_lib.db ${ptpx_design_name}.db
 
-write_interface_timing ${ptpx_design_name}_etm_netlist_interface_timing.report
-
